* Upstream Vagrant Install<br />
Download and execute the vagrant-libvirt-qa install script (installs latest vagrant by default):
```shell
curl -O https://raw.githubusercontent.com/vagrant-libvirt/vagrant-libvirt-qa/main/scripts/install.bash
chmod a+x ./install.bash
./install.bash
```
* Alternatively install vagrant following [https://www.vagrantup.com/downloads](https://www.vagrantup.com/downloads):
{% case include.distro -%}
{%   when "debian", "ubuntu" -%}
```shell
curl -fsSL https://apt.releases.hashicorp.com/gpg | sudo tee /usr/share/keyrings/hashicorp-archive-keyring.asc
echo "deb [ signed-by=/usr/share/keyrings/hashicorp-archive-keyring.asc ] https://apt.releases.hashicorp.com $(lsb_release -cs) main" | sudo tee /etc/apt/sources.list.d/hashicorp.list
sudo apt-get update && sudo apt-get install vagrant
```
{%   when "fedora" -%}
```shell
sudo dnf install -y dnf-plugins-core
sudo dnf config-manager --add-repo https://rpm.releases.hashicorp.com/fedora/hashicorp.repo
sudo dnf -y install vagrant
```
{%   when "centos", "centos8" -%}
```shell
sudo dnf install -y dnf-plugins-core
sudo dnf config-manager --add-repo https://rpm.releases.hashicorp.com/RHEL/hashicorp.repo
sudo dnf -y install vagrant
```
{%   when "centos6", "centos7" -%}
```shell
sudo yum install -y yum-utils
sudo yum-config-manager --add-repo https://rpm.releases.hashicorp.com/RHEL/hashicorp.repo
sudo yum -y install vagrant
```
{%   when "opensuse" %}

  As there is no upstream repository this will not be kept up to date automatically.
  {: .warn }

  ```shell
  version="$(
      wget -qO - https://checkpoint-api.hashicorp.com/v1/check/vagrant 2>/dev/null | \
          tr ',' '\n' | grep current_version | cut -d: -f2 | tr -d '"'
  )"
  wget --no-verbose -O vagrant.rpm \
        https://releases.hashicorp.com/vagrant/${version}/vagrant_${version}_x86_64.rpm
  sudo zypper install --allow-unsigned-rpm --no-confirm vagrant.rpm
  ```
{%   else -%}
Missing the correct distro for vagrant install in site generation
{% endcase -%}
