<interface type='<%= @type %>'<% if @trust_guest_rx_filters %> trustGuestRxFilters='yes'<% end %>>
  <alias name='ua-net-<%= @iface_number %>'/>
  <% if @mac %>
  <mac address='<%= @mac %>'/>
  <% end %>
  <%if @type == 'direct'%>
  <source dev='<%= @device %>' mode='<%= @mode %>'/>
  <% elsif !@portgroup.nil? %>
  <source network='<%=@network_name%>' portgroup='<%=@portgroup%>'/>
  <% else %>
  <source bridge='<%=@device%>'/>
  <% end %>
  <model type='<%=@model_type%>'/>
  <% if @driver_name and @driver_queues %>
    <driver <% if @driver_iommu %> iommu="on" <% end %> name='<%=@driver_name%>' queues='<%=@driver_queues%>'/>
  <% elsif @driver_queues %>
    <driver <% if @driver_iommu %> iommu="on" <% end %> queues='<%=@driver_queues%>'/>
  <% elsif @driver_name %>
    <driver <% if @driver_iommu %> iommu="on" <% end %> name='<%=@driver_name%>'/>
  <% elsif @driver_iommu %>
    <driver iommu='on' />
  <% end %>

  <% if @ovs %>
  <virtualport type='openvswitch'>
    <% if @ovs_interfaceid %>
      <parameters interfaceid='<%=@ovs_interfaceid%>'/>
    <% end %>
  </virtualport>
  <% end %>
  <% if @pci_bus and @pci_slot %>
  <address type='pci' bus='<%=@pci_bus%>' slot='<%=@pci_slot%>' />
  <% end %>
</interface>
